class hottShader(
	color Ks = 1;
	float Km = 1;
	string textureName = "";
	float textureType = 1;
	float Om = 1;
	string opacityName = "";
	float opacityType = 1;
	float ior = 1.33;
	float samples = 8;
	float angle = 0.01;
	float reflection = 0;
	float refraction = 0;
	float extRate = 1;
	float extAmount = 1;
	float Sm = 0;
	float cellFalloff = 0.05;
	float gatherMult = 1;
	float idValue = 0;
	float gammaCorrect = 1;
	string pointcloudName = "[passinfo rmanRenderRadiosityPass filename]";
	varying float __faceindex = 0;
    float dispKm = 1;
	string dispTextureName = "";
    float dispTextureSpace = 1;
	float dispTextureType = 1;
	output varying color occlusion_aov;
	output varying color indirect_aov;
	output varying color lightpass_aov;
	output varying color reflection_aov;
	output varying color refraction_aov;
	output varying color specular_aov;
	output varying color idpass01_aov;
	output varying color idpass02_aov;
	output varying color idpass03_aov;
	output varying color depthAntialiased_aov;
	output varying color depthAliased_aov;
	output varying color wpAntialiased_aov;
	output varying color wpAliased_aov;
) {

	private float gammaCorrectFunc(float x; float gamma;) {
		return pow(x, 1/gamma);
	}
	
	public void displacement(output point P; output normal N) {
		if (dispTextureName != "") {
			
			if (dispTextureSpace == 1) {
			
				color noi;
				
				if (dispTextureType == 1) {
					noi = color ptexture(dispTextureName,0,__faceindex,"blur",0,"width",0.1,"filter","bspline");
				} else {
		        	noi = color texture(dispTextureName,s,t);
				}
				
				point Po = transform("current","object", P) + normal(noi[0],noi[1],noi[2]) * dispKm;
		
		        P = transform("object", "current", Po);
		        N = normalize(calculatenormal(P));
			
			} else {
			
				float noi;
				
				if (dispTextureType == 1) {
					noi = float ptexture(dispTextureName,0,__faceindex,"blur",0,"width",0.1,"filter","bspline");
				} else {
		        	noi = float texture(dispTextureName,s,t);
				}
				
				point Po = transform("current","object", P) + normalize(transform("current","object", N)) * noi * dispKm;
		
		        P = transform("object", "current", Po);
		        N = normalize(calculatenormal(P));
			
			}
			
		}
	}
	
	public void opacity(output color Oi) {
		if (opacityName != "") {
			if (opacityType == 1) {
				Oi = color ptexture(opacityName,0,__faceindex) * Om;
			} else {
				Oi = color texture(opacityName) * Om;
			}
		} else {
			Oi = color(1,1,1) * Om;
		}
	}
	
	public void surface(output color Ci, Oi) {
		uniform float raydepth; // May not be needed
		uniform float frameFloat;
		uniform string pointcloudMap = pointcloudName;
		uniform string passClass;
		float nondiffuse;
		float nonspecular;
		float pointArea;
		color pointRadiosity;
		float gamma = 0.455;
		
		if (textureName != "") {
			if (textureType == 1) {
				Ci = color ptexture(textureName,0,__faceindex);
			} else {
				Ci = color texture(textureName);
			}
		} else {
			Ci = Ks;
		}
		
		Ci *= Km;
		
		if (gammaCorrect == 1) {
			Ci[0] = gammaCorrectFunc(Ci[0], gamma);
			Ci[1] = gammaCorrectFunc(Ci[1], gamma);
			Ci[2] = gammaCorrectFunc(Ci[2], gamma);
		}
		
		Ci *= Oi;
		
		option( "user:pass_class", passClass );
		if (passClass == "RenderRadiosity") {
		
			if (Oi != color(0,0,0)) {
				rayinfo("depth", raydepth); //Depth passsing may be disposable as gather ray lies outside codes scope
				if (raydepth == 0) {
					
					pointRadiosity = Ci * gatherMult;
					pointArea = area(P);
					
					bake3d(pointcloudMap, "_radiosity,_area,Cs", P, normalize(N), "interpolate", 1, "_radiosity", pointRadiosity, "_area", pointArea, "Cs", Ci);
				}
			}
			
		} else if (passClass == "Final") {
			
			normal Nn = normalize(N);
			vector In = normalize(I);
			vector Nf = faceforward(Nn, In);
			float eta = (In.Nn < 0) ? 1/ior : ior;
			vector reflDir, refrDir;
			float kr, kt;
			fresnel(In, Nf, eta, kr, kt, reflDir, refrDir);
			
			color Oc = 0;
			color Ac = 0;
			color Sc = 0;
			vector Ln;
			illuminance( "std", P, Nn, PI/2 )
			{
				Ln = normalize(L);
				
				Oc = Cl;
				Oc *= smoothstep(0,cellFalloff,Ln.Nn);
				
				if (Oc[0] > Ac[0]) {
					Ac[0] = Oc[0];
				}
				
				if (Oc[1] > Ac[1]) {
					Ac[1] = Oc[1];
				}
				
				if (Oc[2] > Ac[2]) {
					Ac[2] = Oc[2];
				}
				
				Oc = Cl;
				Oc *= pow(max(0.0,reflDir.Ln), 1);
				
				if (Oc[0] > Sc[0]) {
					Sc[0] = Oc[0];
				}
				
				if (Oc[1] > Sc[1]) {
					Sc[1] = Oc[1];
				}
				
				if (Oc[2] > Sc[2]) {
					Sc[2] = Oc[2];
				}
			}
			
			illuminance( "occlusion", P )
			{
				occlusion_aov = Cl * Oi;
			}
			
			illuminance( "indirect", P )
			{
				indirect_aov = Cl * Oi;
			}
			
			color hitCi = 0;
			float dist = 0;
			
			if (reflection != 0) {
				reflection_aov = 0;
				gather("illuminance", P, reflDir, (PI/2)*angle, samples, "surface:Ci", hitCi) {
					reflection_aov += (hitCi * kr) / 8;
					reflection_aov *= Oi;
				}
			}
				
			if (refraction != 0) {
				refraction_aov = 0;
				gather("illuminance", P, refrDir, (PI/2)*angle, samples, "ray:length", dist, "surface:Ci", hitCi) {
					if(-In.Nn > 0) {
						hitCi[0] *= exp(-extRate * dist * extAmount);
						hitCi[1] *= exp(-extRate * dist * extAmount);
						hitCi[2] *= exp(-extRate * dist * extAmount);
					}
					refraction_aov += (hitCi * kt) / 8;
					refraction_aov *= Oi;
				}
			}
			
			if (idValue == 1) {
				idpass01_aov = (1,0,0) * Oi;
			} else if (idValue == 2) {
				idpass01_aov = (0,1,0) * Oi;
			} else if (idValue == 3) {
				idpass01_aov = (0,0,1) * Oi;
			} else if (idValue == 4) {
				idpass02_aov = (1,0,0) * Oi;
			} else if (idValue == 5) {
				idpass02_aov = (0,1,0) * Oi;
			} else if (idValue == 6) {
				idpass02_aov = (0,0,1) * Oi;
			} else if (idValue == 7) {
				idpass03_aov = (1,0,0);
			} else if (idValue == 8) {
				idpass03_aov = (0,1,0);
			} else if (idValue == 9) {
				idpass03_aov = (0,0,1);
			}
			
			idpass01_aov *= Oi;
			idpass02_aov *= Oi;
			idpass03_aov *= Oi;
			lightpass_aov = Ac * Oi;
			specular_aov = Sc * Oi;
			depthAntialiased_aov = length(I) * Oi;
			depthAliased_aov = length(I) * Oi;
			wpAntialiased_aov = color(transform("world",P));
			wpAliased_aov = color(transform("world",P));
		}
	}
}
